==> root-ts-1.0.1.tgz: package/src/lib/decorator.ts <==
import { APIRoute } from "./route"
import { Service } from "./service"
export interface RouteDecorator<TThis = any> {
    (this: TThis, obj: any, name: string, descriptor: TypedPropertyDescriptor<RouteInterface<TThis>>): void
}

export interface RouteInterface<TThis = any> {
    (this: TThis, context: TThis, ...args: any[]): void
}

export interface RouteDecoratorFactory<TThis = any, TOption = null> {
    (option: TOption): RouteDecorator<TThis>
}
export namespace RouteDecoratorFactory {
    export function from<TThis = any, TOption = null>(guard: GuardFunction<TThis, TOption>): RouteDecoratorFactory<TThis, TOption> {
        return function (option: TOption) {
            return function (this: TThis, obj: TThis, name: string, descriptor: TypedPropertyDescriptor<RouteInterface<TThis>>) {
                let orig = descriptor.value
                descriptor.value = function (this: TThis, context: TThis) {
                    guard.call(this, this, option, () => {
                        orig.call(this, this)
                    })
                }
            }
        }
    }
    export const generator = from
    export function decorate<TThis = any>(solve: SolveFunction<TThis>): RouteDecorator<TThis> {
        return function (this: TThis, obj: TThis, name: string, descriptor: TypedPropertyDescriptor<RouteInterface<TThis>>) {
            let orig = descriptor.value
            descriptor.value = function (this: TThis, context: TThis) {
                solve.call(this, this, null, () => {
                    orig.call(this, this)
                })
            }
        }
    }
}
export interface GuardFunction<TThis = any, TOption = any> {
    (this: TThis, context: TThis, option: TOption, pass: () => void): void
}
export interface SolveFunction<TThis = any> {
    (this: TThis, context: TThis, pass: () => void): void
}
export interface FakeService<TServices> extends Service {
    services: TServices
}
export class RouteDecorators<TSpecs extends RouteDecoratorSpecs = any> {
    constructor(public spec: TSpecs) {
    }
    private decs: {
        [K in keyof TSpecs]?: RouteDecoratorFactory
    } = {}
    implement<K extends keyof TSpecs>(name: K, guard: GuardFunction<APIRoute<TSpecs[K]["return"], Error, FakeService<TSpecs[K]["services"]>> & TSpecs[K], TSpecs[K]["option"]>) {
        if (this.decs[name]) {
            throw new Error("Duplication implementation of decorator " + name)
        }
        this.decs[name] = RouteDecoratorFactory.from(guard)
    }
    decorate<K extends keyof TSpecs>(name: K, option?: TSpecs[K]["option"]) {
        let factory = this.decs[name]
        if (!factory) {
            throw new Error(`Implementation of decorator ${name} not found`)
        }
        return factory(option)
    }
}

export abstract class RouteDecoratorSpec {
    params?
    services?
    option?
    return?
}
export interface RouteDecoratorSpecs {
    [name: string]: RouteDecoratorSpec
}

==> root-ts-1.0.14.tgz: package/src/lib/decorator.ts <==
import { APIRoute } from "./route"
import { Service } from "./service"
export interface RouteDecorator<TThis = any> {
    (this: TThis, obj: any, name: string, descriptor: TypedPropertyDescriptor<RouteInterface<TThis>>): void
}

export interface RouteInterface<TThis = any> {
    (this: TThis, context: TThis, ...args: any[]): void
}

export interface RouteDecoratorFactory<TThis = any, TOption = null> {
    (option: TOption): RouteDecorator<TThis>
}
export namespace RouteDecoratorFactory {
    export function from<TThis = any, TOption = null>(guard: GuardFunction<TThis, TOption>): RouteDecoratorFactory<TThis, TOption> {
        return function (option: TOption) {
            return function (this: TThis, obj: TThis, name: string, descriptor: TypedPropertyDescriptor<RouteInterface<TThis>>) {
                let orig = descriptor.value
                descriptor.value = function (this: TThis, context: TThis) {
                    guard.call(this, this, option, () => {
                        orig.call(this, this)
                    })
                }
            }
        }
    }
    export const generator = from
    export function decorate<TThis = any>(solve: SolveFunction<TThis>): RouteDecorator<TThis> {
        return function (this: TThis, obj: TThis, name: string, descriptor: TypedPropertyDescriptor<RouteInterface<TThis>>) {
            let orig = descriptor.value
            descriptor.value = function (this: TThis, context: TThis) {
                solve.call(this, this, () => {
                    orig.call(this, this)
                })
            }
        }
    }
}
export interface GuardFunction<TThis = any, TOption = any> {
    (this: TThis, context: TThis, option: TOption, pass: () => void): void
}
export interface SolveFunction<TThis = any> {
    (this: TThis, context: TThis, pass: () => void): void
}
export interface FakeService<TServices> extends Service {
    services: TServices
}
export class RouteDecorators<TSpecs extends RouteDecoratorSpecs = any> {
    constructor(public spec: TSpecs) {
    }
    private decs: {
        [K in keyof TSpecs]?: RouteDecoratorFactory
    } = {}
    implement<K extends keyof TSpecs>(name: K, guard: GuardFunction<APIRoute<TSpecs[K]["return"], Error, FakeService<TSpecs[K]["services"]>> & TSpecs[K], TSpecs[K]["option"]>) {
        if (this.decs[name]) {
            throw new Error("Duplication implementation of decorator " + name)
        }
        this.decs[name] = RouteDecoratorFactory.from(guard)
    }
    decorate<K extends keyof TSpecs>(name: K, option?: TSpecs[K]["option"]) {
        let factory = this.decs[name]
        if (!factory) {
            throw new Error(`Implementation of decorator ${name} not found`)
        }
        return factory(option)
    }
}

export abstract class RouteDecoratorSpec {
    params?
    services?
    option?
    return?
}
export interface RouteDecoratorSpecs {
    [name: string]: RouteDecoratorSpec
}
